/****************************************************************************/
/*                  JL-080 sample programs for C1 Master                    */
/*                                                                          */
/*    This file contains sample programs of                                 */
/*      (1) mst_init(void) : JL-080 initialization for MECHATROLINK-II,     */
/*      (2) mst_exchange(void) : Send/recieve data processing.              */
/*                                                                          */
/*    These sample programs are made under the following conditions;        */
/*                   Number of Slaves : 7                                   */
/*                 Transmission Cycle : 1 msec                              */
/*                Communication Cycle : 1 msec                              */
/*      JL-080 register start address : 0xDE000000                          */
/*                        packet size : 32 [bytes]                          */
/*             Max. number of Retries : 1                                   */
/*            Time slot for C2 Master : 1(use)                              */
/*                                                                          */
/*    Please refer to the following document for more detials.              */
/*       "MECHATROLINK-II JL-080 Access Driver Instructions                 */
/*                                            for Master Station"           */
/*                                                                          */
/*    == IMPORTANT ==                                                       */
/*      The interface registers of the JL-080                               */
/*      handle unsigned short data in the sample programs.                  */
/*      When using an implementation in which unsigned short data is not    */
/*      16 bits, it will be necessary to change the part of the source code */
/*      that accesses the interface registers.                              */
/*                                                                          */
/*                                          Yaskawa Electric Corporation    */
/*                                                                          */
/****************************************************************************/


/* includes */
#include "jl080gbl.h"

/* defines */
#define LINK_SIZE			16			/* Link data size [in words] 16 means 32 bytes */
#define SLV_ST_MAX			7			/* Maximum Number of Slaves */
#define COM_TIME			1			/* Communication Cycle[multiple of Transmission Cycle] */
#define HOST_IF_REGS_ADDR	0xDE000000	/* JL-080 register start address */

/* globals */
CHANNEL_INFO	chbuff;								/* Channel Buffer */
USHORT			lnk_sbuff[SLV_ST_MAX][LINK_SIZE];	/* Send Buffer */
USHORT			lnk_rbuff[SLV_ST_MAX][LINK_SIZE];	/* Receive Buffer */
USHORT			lnk_sctrl[SLV_ST_MAX];				/* Send Link Header  */
USHORT			lnk_rctrl[SLV_ST_MAX];				/* Receive Link Header */
UINT			err_axis;							/* Staion # which error occured */

/* forward declarations */
int mst_init(void);						/* Initialize MECHATROLINK communicaiton(Setup JL-080) */
int mst_exchange(void);					/* Send/Recieve Link data in cyclic communication */
void Set_st_addr(UCHAR* pst);			/* Set Station addresses */

/****************************************************************************/
/*                                                                          */
/* Name :                                                                   */
/*        Sample of JL-080 Initialization procedure                         */
/* Function:                                                                */
/*       Initializing JL-080                                                */
/* Conditions:                                                              */
/*                  Protocol : MECHATROLINK-II(2:N Communication)           */
/*               Buffer mode : Fixed buffer mode                            */
/*                     Speed : 10Mbps                                       */
/*        Transmission cycle : 1ms                                          */
/*               Packet Size : 32[byte]                                     */
/*       C2 Master Time slot : Use time slot for C2 master                  */
/*                                                                          */
/* Return Code :                                                            */
/*      OK :                            Normal End                          */
/*      ERROR_INIT_CHANNEL :            Invalid channel buffer specified    */
/*      ERROR_POLL_TIMEOUT :            Request did not completed           */
/*      ERROR_NOT_READY :               JL-080 Not Ready (STS(RDY) is zero.)*/
/*      ERROR_INVALID_PROTOCOL :        Unsupported protocol                */
/*      ERROR_INVALID_BMODE :           Unsupported buffer mode             */
/*      ERROR_POLL_TIMEOUT_REG :        P_RSTAK Request did not completed   */
/*      ERROR_POLL_TIMEOUT_PAR :        PAR_I Request did not completed     */
/*      ERROR_POLL_TIMEOUT_ENA :        Request did not completed           */
/****************************************************************************/
int mst_init(void)
{
	CHANNEL_INFO*	chbufp;		/* Channel Buffer */
	YENET_USER_PAR	usr_par;	/* Communication Parameters */
	UCHAR			st_addr[SLV_ST_MAX+1];	/* Slave Addresses */
	int				ret = 0;	/* return code */

	chbufp = &chbuff;	/* Get the pointer of Channel Buffer */

/* Check JL-080 RAM */
/* Read/Write check 2048 words from address 0 */
	if((ret = yns_check_ram((USHORT*)HOST_IF_REGS_ADDR, 0, 2048, 0x5aa5)) != OK)
		return ret;
	if((ret = yns_check_ram((USHORT*)HOST_IF_REGS_ADDR, 0, 2048, 0xa55a)) != OK)
		return ret;
	if((ret = yns_check_ram((USHORT*)HOST_IF_REGS_ADDR, 0, 2048, 0x0000)) != OK)
		return ret;

/* Setup parameters of JL-080 */
	usr_par.mod = YN_MOD_PROCEL_2NS | YN_MOD_SPEED_10M | YN_MOD_BMOD_STATIC
				| YN_MOD_TYPE_MASTER | YN_MOD_INT_FRS;

	usr_par.ma = 0x01;			/* My Address(C1 Master) */
	usr_par.ma_max = SLV_ST_MAX;/* Maximum Number of Slaves */
	usr_par.t_mcyc = 4000;		/* Transmission cycle[0.25us] */
	usr_par.t_cyc = 360;		/* Time slot[0.25us] */
	usr_par.byte = 32;			/* Packet size */
	usr_par.dev = 0;			/* INT1 dividing ratio(for INT2 output) */
	usr_par.max_rtry = 1;		/* Max. number of Retries per Transmission cycle */
	usr_par.c2m_ch = 1;			/* Time slot for C2 Master(0:not use, 1:use) */
	usr_par.wdt = 15;			/* Watch dog timer [10ms]; if wdt=0, WDT function disabled */

/* Setup Station addresses */
	Set_st_addr(st_addr);

/* JL-080 Initialization */
	ret = yns_sequent_init(chbufp, (USHORT*)HOST_IF_REGS_ADDR, &usr_par, st_addr);

	return ret;
}


/****************************************************************************/
/*                                                                          */
/* Name :                                                                   */
/*       Sample of send/recieve data procedure                              */
/*                                                                          */
/* Function :                                                               */
/*      Setting data in the lnk_sbuff to JL-080, and saving received data   */
/*      to the lnk_rbuff.                                                   */
/*                                                                          */
/* Return Code:                                                             */
/*      OK :                          Normal End                            */
/*      ERROR_ALT_BUFFER :            Time out of Toggle buffer             */
/*      ERROR_INVALID_ST_NO :         Invalid Channel No.                   */
/*      ERROR_RECV_DATA:              Receive error                         */
/*      ERROR_JL080_APPL:             JL-080 status error(user)             */
/*      ERROR_JL080_SYSTEM:           JL-080 status error(system)           */
/****************************************************************************/
int mst_exchange(void)
{
	CHANNEL_INFO*	chbufp;		/* Channel Buffer */
	USHORT			ests;		/* Error Status */
	int				ch;			/* Channel No. */
	int				ret = OK, rc;/* return code */

	chbufp = &chbuff;	/* Set pointer of Channel Buffer */

/* Start cyclic communication */
	if((ret = yns_begin_link_acc(chbufp)) != OK)
	{
	/* Error detection */
		yns_set_netclk(chbufp, COM_TIME);	/* Update Network time-stamp (for C2-Master) */
		yns_end_link_acc(chbufp);			/* Terminate cyclic communication */
		return ret;
	}

/* JL-080 status error(ESTS) check */
	if((ests = yns_read_comstat(chbufp)) != 0)
	{
		if(ests & YN_ESTS_SYS_ERR)
		{	/* JL-080 System error!! */
			yns_end_link_acc(chbufp);
			return ERROR_JL080_SYSTEM;	/* Stop cyclic communication */
		}
		else
		{	/* JL-080 Application(user) error!! */
			yns_end_link_acc(chbufp);
			return ERROR_JL080_APPL;	/* Stop cyclic communication */
		}
	}

/* Send/Receive Process for all stations(slaves) */
	for(ch = 0; ch < chbufp->ma_max; ch++)
	{
	/* Send [buffer->JL-080] */
		yns_write_ldata(chbufp, ch+1, lnk_sbuff[ch], LINK_SIZE);
/*		yns_write_ldata2(chbufp, ch+1, lnk_sctrl[ch], lnk_sbuff[ch], LINK_SIZE); Send data with CTRL code */

	/* Receive [JL-080->buffer] */
		if((rc = yns_read_ldata(chbufp, 
				ch+1, lnk_rbuff[ch], LINK_SIZE)) < 0)
/*		if((rc = yns_read_ldata2(chbufp, 
				ch+1, &lnk_rctrl[ch], lnk_rbuff[ch], LINK_SIZE)) < 0) Receive data with CTRL code */
		{
		/* Error detection */
			err_axis |= (0x1 << ch);	/* set station number which occurred receive error */
			ret = rc;
		}
	}

/* Finish link read/write access */
	yns_set_netclk(chbufp, COM_TIME);	/* Update Network Time-stamp (for C2-Master) */
	yns_end_link_acc(chbufp);

	return ret;
}


/****************************************************************************/
/*                                                                          */
/* Name :                                                                   */
/* Set station address                                                      */
/*                                                                          */
/* Function :                                                               */
/*      Convert station number to station address                           */
/* Conditions:                                                              */
/*      Setting all servodrive addresses or 41H - 41H + SLV_ST_MAX          */
/*                                                                          */
/****************************************************************************/
void Set_st_addr(UCHAR* pst)
{
	int	ch;

	pst[0] = 0xFF;	/* Global address(for Sync frame) */
	for(ch = 1; ch <= SLV_ST_MAX; ch++)
	{
		pst[ch] = 0x40 + ch;	/* Station address(for Servo drive) */
		lnk_sctrl[ch-1] = (YN_CTRL_CDRW << 8) | pst[ch];	/* Set Link Header */
	}
}

